@@ -24,13 +24,13 @@ else:
 
 class Martini():
     """
-    Used for the creation of synthetic HI data cubes from simulation data.
+    Creates synthetic HI data cubes from simulation data.
 
     Usual use of martini involves first creating instances of classes from each
     of the required and optional sub-modules, then creating a Martini with
     these instances as arguments. The object can then be used to create
-    synthetic observations, usually by calling 'insert_source_in_cube',
-    (optionally) 'add_noise', (optionally) 'convolve_beam' and 'write_fits' in
+    synthetic observations, usually by calling `insert_source_in_cube`,
+    (optionally) `add_noise`, (optionally) `convolve_beam` and `write_fits` in
     order.
 
     Parameters
@@ -48,21 +48,20 @@ class Martini():
         A description of the datacube to create, including pixels, channels,
         sky position. See sub-module documentation.
 
-    beam : (optional) an instance of a class derived from beams._BaseBeam
+    beam : an instance of a class derived from beams._BaseBeam, optional
         A description of the beam for the simulated telescope. Given a
         description, either mathematical or as an image, the creation of a
         custom beam is straightforward. See sub-module documentation.
 
-    noise : (optional) an instance of a class derived from noise._BaseNoise
+    noise : an instance of a class derived from noise._BaseNoise, optional
         A description of the simulated noise. A simple Gaussian noise model is
         provided; implementation of other noise models is straightforward. See
         sub-module documentation.
 
     sph_kernel : an instance of a class derived from sph_kernels._BaseSPHKernel
-        A description of the SPH smoothing kernel. The Wendland C2 kernel (used
-        in EAGLE), and a point-like Dirac-delta kernel are implemented,
-        implementation of other kernels is straightforward. See sub-module
-        documentation.
+        A description of the SPH smoothing kernel. Check simulation
+        documentation for the kernel used in a particular simulation, and
+        SPH kernel submodule documentation for guidance.
 
     spectral_model : an instance of a class derived from \
     spectral_models._BaseSpectrum
@@ -74,11 +73,6 @@ class Martini():
     logtag : string
         String to prepend to standard output messages.
 
-    Returns
-    -------
-    out : Martini
-        A Martini object configured with the specified sub-modules.
-
     See Also
     --------
     martini.sources
@@ -90,6 +84,9 @@ class Martini():
 
     Examples
     --------
+    More detailed examples can be found in the examples directory in the github
+    distribution of the package.
+
     The following example illustrates basic use of martini, using a (very!)
     crude model of a gas disk. This example can be run by doing
     'from martini import demo; demo()'::
@@ -187,160 +184,6 @@ class Martini():
         M.write_beam_fits('testbeam.fits', channels='velocity')
         M.write_fits('testcube.fits', channels='velocity')
 
-    The following example illustrates the usage of Martini for an IllustrisTNG
-    source. The example can be run in the TNG JupyterLab environment, or in a
-    local environment configured following the fiducial TNG setup
-    (http://www.tng-project.org/data/docs/scripts/). It can be run by doing
-    'from martini import tngdemo; tngdemo()'::
-
-        from martini.sources import TNGSource
-        from martini import DataCube, Martini
-        from martini.beams import GaussianBeam
-        from martini.noise import GaussianNoise
-        from martini.spectral_models import GaussianSpectrum
-        from martini.sph_kernels import GaussianKernel, CubicSplineKernel
-        import astropy.units as U
-
-        # This example runs in about 1 minute in the IllustrisTNG JupyterLab
-        # environment.
-
-        # Parameters myBasePath, mySnap and myId follow the usual TNG
-        # conventions as in the illustris_python package.
-        myBasePath = 'sims.TNG/TNG100-1/output/'
-        mySnap = 99
-        myId = 385350  # first central with 218 < Vmax < 220, and SFR > 1
-
-        # The different martini sub-modules need to be initialized, see
-        # https://kyleaoman.github.io/martini/build/html/martini.html for a
-        # high-level overview. See the documentation for the individual sub-
-        # modules for details of all configuration options. Comments below
-        # highlight a few suggested best-practices specific to TNG.
-
-        # SOURCE
-        # The rotation configuration takes an inclination (here 60deg) and
-        # rotation about the pole (here 0deg). The code attempts to
-        # automatically align the galactic disk in the y-z plane by aligning
-        # the angular momentum along the x-axis. The polar rotation is then
-        # applied, and finally the disc inclined by a rotation around the
-        # y-axis (the line of sight is along the x-axis). The automatic
-        # alignment will work for typical reasonably isolated discs, but will
-        # struggle when companions are present, when the angular momentum axis
-        # is a poor tracer of the disc plane, and especially for satellites. If
-        # finer control of the orientation is needed, derive the transformation
-        # from the simulation box coordinates to the desired coordinates for
-        # the 'observation', keeping in mind that the line of sight is along
-        # the x-axis. This rotation matrix can then be passed to rotation as
-        # {'rotmat': np.eye(3)} (here the identity rotation matrix used as an
-        # example). A common problem in this case is deriving the inverse
-        # transform instead of the forward transform, if unexpected results are
-        # obtained, first try passing the transpose of the rotation matrix.
-        # Note that initializing the source takes some time as the particle
-        # data must be read from disk.
-        source = TNGSource(
-            myBasePath,
-            mySnap,
-            myId,
-            distance=30 * U.Mpc,
-            rotation={'L_coords': (60 * U.deg, 0. * U.deg)},
-            ra=0. * U.deg,
-            dec=0. * U.deg
-        )
-
-        # DATACUBE
-        # It is usually advisable to set the centre of the cube to track the
-        # centre of the source, as illustrated below. Note that the source
-        # systemic velocity is set according to the distance and Hubble's law.
-        # These values can instead be set explicitly, if desired. A datacube
-        # with 128x128 pixels usually takes a few minutes, 1024x1024 could take
-        # several hours. The number of channels has less influence on the
-        # runtime. Most of the runtime is spent when M.insert_source_in_cube is
-        # called below.
-        datacube = DataCube(
-            n_px_x=128,
-            n_px_y=128,
-            n_channels=32,
-            px_size=10. * U.arcsec,
-            channel_width=10. * U.km * U.s ** -1,
-            velocity_centre=source.vsys,
-            ra=source.ra,
-            dec=source.dec
-        )
-
-        # BEAM
-        # It is usually advisable to set the beam size to be ~3x the pixel
-        # size. Note that the data cube is padded according to the size of the
-        # beam, this usually results in the number of pixel rows printed in the
-        # progress messages to differ from the requested dimensions. The
-        # padding is required for accurate convolution with the beam, but
-        # contains incorrect values after convolution and is discarded to
-        # produce the final data cube of the requested size.
-        beam = GaussianBeam(
-            bmaj=30. * U.arcsec,
-            bmin=30. * U.arcsec,
-            bpa=0. * U.deg,
-            truncate=3.
-        )
-
-        # NOISE
-        # The noise is normally added before convolution with the beam (as
-        # below in this example). The rms value passed is for the noise before
-        # convolution, the rms noise in the output data cube will therefore
-        # typically differ from this value.
-        noise = GaussianNoise(
-            rms=5.E-6 * U.Jy * U.arcsec ** -2
-        )
-
-        # SPECTRAL MODEL
-        # The 'subgrid' velocity dispersion can also be fixed to a constant
-        # value, e.g. sigma=7 * U.km * U.s**-1.
-        spectral_model = GaussianSpectrum(
-            sigma='thermal'
-        )
-
-        # SPH KERNEL
-        # Since IllustrisTNG uses a moving mesh hydrodynamics solver (Arepo),
-        # there are no formal SPH smoothing lengths and no specified kernel.
-        # However, approximate smoothing lengths are provided by Subfind, so a
-        # reasonable approximation is to use these for imaging. The lengths
-        # correspond roughly to a cubic spline kernel. The implementation of
-        # the cubic spline kernel included in Martini uses an approximation
-        # which breaks down when the particle smoothing lengths are small
-        # compared to the size of the pixels (at the distance of the source).
-        # The Gaussian kernel implementation does not suffer from this
-        # limitation and can be scaled to mimic the cubic spline kernel as
-        # illustrated below. This solution should generally work; for
-        # specialized applications, consider making a histogram of the
-        # smoothing lengths of the particles in the source (in units of the
-        # pixel scale) and consulting the documentation of the sph_kernels sub-
-        # module to select an appropriate kernel.
-        sph_kernel = GaussianKernel.mimic(
-            CubicSplineKernel(rescale_sph_h=0.5),
-            truncate=3
-        )
-
-        M = Martini(
-            source=source,
-            datacube=datacube,
-            beam=beam,
-            noise=noise,
-            spectral_model=spectral_model,
-            sph_kernel=sph_kernel
-        )
-
-        # Progress messages will be printed every printfreq rows; suppress by
-        # setting to None.
-        M.insert_source_in_cube(printfreq=1)
-        M.add_noise()
-        M.convolve_beam()
-
-        # Two output formats are available, depending on preference. Both
-        # formats are self-documenting, via FITS header keywords and HDF5
-        # attributes, respectively. For HDF5 output, the beam image is included
-        # in the same file.
-        M.write_fits('tngdemo.fits', channels='velocity')
-        M.write_beam_fits('tngdemo_beam.fits', channels='velocity')
-        M.write_hdf5('tngdemo.hdf5', channels='velocity')
-
     """
 
     def __init__(self,
@@ -444,6 +287,8 @@ class Martini():
         for condition in reject_conditions:
             reject_mask = np.logical_or(reject_mask, condition)
         self.source.apply_mask(np.logical_not(reject_mask))
+        # most kernels ignore this line, but required by AdaptiveKernel
+        self.sph_kernel.apply_mask(np.logical_not(reject_mask))
         return
 
     def insert_source_in_cube(self, skip_validation=False, printfreq=100):
@@ -452,17 +297,17 @@ class Martini():
 
         Parameters
         ----------
-        skip_validation : bool
+        skip_validation : bool, optional
             SPH kernel interpolation onto the DataCube is approximated for
             increased speed. For some combinations of pixel size, distance
             and SPH smoothing length, the approximation may break down. The
             kernel class will check whether this will occur and raise a
             RuntimeError if so. This validation can be skipped (at the cost
-            of accuracy!) by setting this parameter True.
+            of accuracy!) by setting this parameter True. (Default: False.)
 
-        printfreq : int or None
+        printfreq : int or None, optional
             Every printfreq rows a message will be printed to track progress.
-            Messages completely suppressed with printfreq=None.
+            Messages completely suppressed with printfreq=None. (Default: 100.)
         """
 
         # pixels indexed from 0 (not like in FITS!) for better use with numpy
@@ -474,11 +319,11 @@ class Martini():
                 skycoords.dec.to(self.datacube.units[1]),
                 skycoords.radial_velocity.to(self.datacube.units[2]),
                 origin)) * U.pix
+        # note that source hsm_g values are required to be FWHM values
         sm_length = np.arctan(
             self.source.hsm_g / self.source.sky_coordinates.distance).to(
                 U.pix, U.pixel_scale(self.datacube.px_size / U.pix))
-        if not skip_validation:
-            self.sph_kernel.confirm_validation(sm_length)
+        self.sph_kernel.confirm_validation(sm_length, noraise=skip_validation)
         sm_range = np.ceil(
             sm_length * self.sph_kernel.size_in_fwhm
         ).astype(int)
@@ -505,7 +350,8 @@ class Martini():
             mask = (np.abs(ij - particle_coords[:2]) <= sm_range).all(axis=0)
             weights = self.sph_kernel.px_weight(
                 particle_coords[:2, mask] - ij,
-                sm_length[mask]
+                sm_length[mask],
+                mask=mask  # most kernels ignore, required by AdaptiveKernel
             )
             self.datacube._array[ij_px[0], ij_px[1], :, 0] = \
                 (self.spectral_model.spectra[mask] *
@@ -525,12 +371,12 @@ class Martini():
             Name of the file to write. '.fits' will be appended if not already
             present.
 
-        channels : 'frequency' (default), or 'velocity'
+        channels : {'frequency', 'velocity'}, optional
             Type of units used along the spectral axis in output file.
+            (Default: 'frequency'.)
 
-        overwrite: bool
-            Whether to allow overwriting existing files, note that the default
-            is True.
+        overwrite: bool, optional
+            Whether to allow overwriting existing files. (Default: True.)
         """
 
         self.datacube.drop_pad()
@@ -627,12 +473,12 @@ class Martini():
             Name of the file to write. '.fits' will be appended if not already
             present.
 
-        channels : 'frequency' (default), or 'velocity'
+        channels : {'frequency', 'velocity'}, optional
             Type of units used along the spectral axis in output file.
+            (Default: 'frequency'.)
 
-        overwrite: bool
-            Whether to allow overwriting existing files, note that the default
-            is True.
+        overwrite: bool, optional
+            Whether to allow overwriting existing files. (Default: True.)
 
         Raises
         ------
@@ -719,21 +565,21 @@ class Martini():
             Name of the file to write. '.hdf5' will be appended if not already
             present.
 
-        channels : 'frequency' (default), or 'velocity'
+        channels : {'frequency', 'velocity'}, optional
             Type of units used along the spectral axis in output file.
+            (Default: 'frequency'.)
 
-        overwrite: bool
-            Whether to allow overwriting existing files, note that the default
-            is True.
+        overwrite: bool, optional
+            Whether to allow overwriting existing files. (Default: True.)
 
-        memmap: bool
+        memmap: bool, optional
             If True, create a file-like object in memory and return it instead
-            of writing file to disk.
+            of writing file to disk. (Default: False.)
 
-        compact: bool
+        compact: bool, optional
             If True, omit pixel coordinate arrays to save disk space. In this
             case pixel coordinates can still be reconstructed from FITS-style
-            keywords stored in the FluxCube attributes. Default is False.
+            keywords stored in the FluxCube attributes. (Default: False.)
         """
 
         import h5py
