@@ -1,7 +1,8 @@
 from ._sph_source import SPHSource
-from ._single_particle_source import SingleParticleSource
-from ._cross_source import CrossSource
+from ._single_particle_source import _SingleParticleSource
+from ._cross_source import _CrossSource
 from ._tng_source import TNGSource
 from ._eagle_source import EAGLESource
 from ._magneticum_source import MagneticumSource
+from ._simba_source import SimbaSource
 from ._so_source import SOSource
