@@ -7,23 +7,28 @@ from warnings import warn
 
 
 def find_fwhm(f):
-    return 2 * fsolve(lambda q: f(q) - f(np.zeros(1)) / 2, .5)
+    return 2 * fsolve(lambda q: f(q) - f(np.zeros(1)) / 2, .5)[0]
 
 
 class _BaseSPHKernel(object):
     """
     Abstract base class for classes implementing SPH kernels to inherit from.
+
     Classes inheriting from _BaseSPHKernel must implement three methods:
-    'kernel', 'kernel_integral' and 'validate'.
-    'kernel' should define the kernel function, normalized such that its volume
+    `kernel`, `kernel_integral` and `validate`.
+
+    `kernel` should define the kernel function, normalized such that its volume
     integral is 1.
-    'kernel_integral' should define the integral of the kernel over a pixel
+
+    `kernel_integral` should define the integral of the kernel over a pixel
     given the distance between the pixel centre and the particle centre, and
     the smoothing length (both in units of pixels). The integral should be
     normalized so that evaluated over the entire kernel it is equal to 1.
-    'validate' should check whether any approximations converge to sufficient
+
+    `validate` should check whether any approximations converge to sufficient
     accuracy (for instance, depending on the ratio of the pixel size and
-    smoothing length), and raise an error if not.
+    smoothing length), and raise an error if not. It should return a boolean
+    array with True for particles which pass validation, and False otherwise.
     """
 
     __metaclass__ = ABCMeta
@@ -39,50 +44,82 @@ class _BaseSPHKernel(object):
              ' updated accordingly.')
         return
 
-    def px_weight(self, dij, h):
+    def px_weight(self, dij, h, mask=None):
         """
         Calculate kernel integral using scaled smoothing lengths.
+
         This is the method that should be called by other modules in
         martini, rather than 'kernel_integral'.
+
         Parameters
         ----------
-        dij : astropy.units.Quantity, with dimensions of pixels
+        dij : Quantity, with dimensions of pixels
             Distances from pixel centre to particle positions, in pixels.
-        h : astropy.units.Quantity, with dimensions of pixels
+        h : Quantity, with dimensions of pixels
             Particle smoothing lengths, defined as the kernel FWHM, in pixels.
+
         Returns
         -------
-        out : astropy.units.Quantity, with dimensions of pixels^-2
+        out : Quantity, with dimensions of pixels^-2
             Integral of smoothing kernel over pixel, per unit pixel area.
         """
+        try:
+            rescaled_h = h * self._rescale[mask]
+        except TypeError:
+            rescaled_h = h * self._rescale
+        return self.kernel_integral(
+            dij,
+            rescaled_h,
+            mask=mask
+        )
 
-        return self.kernel_integral(dij, h * self._rescale)
-
-    def confirm_validation(self, sm_lengths):
+    def confirm_validation(self, sm_lengths, noraise=False):
         """
         Verify kernel accuracy using scaled smoothing lengths.
+
         This is the method that should be called by other modules in
         martini, rather than 'validate'.
+
         Parameters
         ----------
-        sm_lengths : astropy.units.Quantity, with dimensions of pixels
+        sm_lengths : Quantity, with dimensions of pixels
             Particle smoothing lengths, in units of pixels.
         """
 
-        return self.validate(sm_lengths * self._rescale)
+        return self.validate(sm_lengths * self._rescale, noraise=noraise)
+
+    def _validate_error(self, msg, sm_lengths, valid, noraise=False):
+        print('Median smoothing length: ', np.median(sm_lengths), 'px')
+        print('Minimum smoothing length: ', np.min(sm_lengths), 'px')
+        print('Maximum smoothing length: ', np.max(sm_lengths), 'px')
+        print(
+            'Smoothing length histogram (np.histogram):',
+            np.histogram(sm_lengths)
+        )
+        print(
+            np.sum(np.logical_not(valid)),
+            '/',
+            sm_lengths.size,
+            'smoothing lengths fail validation.'
+        )
+        if not noraise:
+            raise RuntimeError(msg)
+        return
 
     def eval_kernel(self, r, h):
         """
         Evaluate the kernel, handling array casting and rescaling.
+
         Parameters
         ----------
-        r : float or np.array or astropy.units.Quantity
+        r : array_like or Quantity
             Distance parameter, same units as h.
-        h : float or np.array or astropy.units.Quantity
+        h : array_like or Quantity
             Smoothing scale parameter (FWHM), same units as r.
+
         Returns
         -------
-        out : float or np.array
+        out : array_like
             Kernel value at position(s) r / h.
         """
 
@@ -97,52 +134,74 @@ class _BaseSPHKernel(object):
         else:
             return W
 
+    def apply_mask(self, mask):
+        """
+        If kernel contains maskable attributes, apply a mask.
+
+        Parameters
+        ----------
+        mask : array_like containing booleans
+            Mask to apply to any maskable attributes.
+        """
+        pass
+
     @abstractmethod
     def kernel(self, q):
         """
         Abstract method; evaluate the kernel.
+
         Parameters
         ----------
-        q : np.array
+        q : array_like
             Dimensionless distance parameter.
+
         Returns
         -------
-        out : np.array
+        out : array_like
             Kernel value at positions q.
         """
         pass
 
     @abstractmethod
-    def kernel_integral(self, dij, h):
+    def kernel_integral(self, dij, h, **kwargs):
         """
         Abstract method; calculate the kernel integral over a pixel.
+
+        Should also accept **kwargs, as a mask (which is generally not
+        needed for the calculation, but used in the AdaptiveKernel) will
+        also be passed.
+
         Parameters
         ----------
-        dij : astropy.units.Quantity, with dimensions of pixels
+        dij : Quantity, with dimensions of pixels
             Distances from pixel centre to particle positions, in pixels.
-        h : astropy.units.Quantity, with dimensions of pixels
+        h : Quantity, with dimensions of pixels
             Particle smoothing lengths (FWHM), in pixels.
+
         Returns
         -------
-        out : astropy.units.Quantity, with dimensions of pixels^-2
+        out : Quantity, with dimensions of pixels^-2
             Integral of smoothing kernel over pixel, per unit pixel area.
         """
 
         pass
 
     @abstractmethod
-    def validate(self, sm_lengths):
+    def validate(self, sm_lengths, noraise=False):
         """
         Abstract method; check conditions for validity of kernel integral
         calculation.
+
         Some approximations may only converge if the ratio of the pixel size
         and the smoothing length is sufficiently large, or sufficiently small.
         This method should check these conditions and raise errors when
         appropriate. The smoothing lengths are provided normalized to the pixel
-        size.
+        size. AdaptiveKernel needs to force errors not to raise, other classes
+        should just provide **kwargs.
+
         Parameters
         ----------
-        sm_lengths : astropy.units.Quantity, with dimensions of pixels
+        sm_lengths : Quantity, with dimensions of pixels
             Particle smoothing lengths, in units of pixels.
         """
 
@@ -152,20 +211,22 @@ class _BaseSPHKernel(object):
 class WendlandC2Kernel(_BaseSPHKernel):
     """
     Implementation of the Wendland C2 kernel integral.
+
     The Wendland C2 kernel is used in the EAGLE code and derivatives (not in
     Gadget/Gadget2!). The exact integral is usually too slow to be practical;
     the implementation here approximates the kernel amplitude as constant
     across the pixel, which converges to within 1% of the exact integral
-    provided the SPH smoothing lengths are at least 2 pixels in size.
+    provided the SPH smoothing lengths are at least 1.51 pixels in size.
+
     The WendlandC2 kernel is here defined as:
-        W(q) = (21 / (2 * pi)) * (1 - q)^4 * (4 * q + 1)
-        for 0 <= q < 1
-        W(q) = 0
-        for q >= 1
-    Returns
-    -------
-    out : WendlandC2Kernel
-        An appropriately initialized WendlandC2Kernel object.
+
+    .. math::
+        W(q) = \\begin{cases}
+        \\frac{21}{2\\pi}(1-q)^4(4q+1)
+        &{\\rm for}\\;0 \\leq q < 1\\\\
+        0 &{\\rm for}\\;q \\geq 1
+        \\end{cases}
+
     """
 
     def __init__(self):
@@ -179,18 +240,24 @@ class WendlandC2Kernel(_BaseSPHKernel):
     def kernel(self, q):
         """
         Evaluate the kernel function.
+
         The WendlandC2 kernel is here defined as:
-        W(q) = (21 / (2 * pi)) * (1 - q)^4 * (4 * q + 1)
-        for 0 <= q < 1
-        W(q) = 0
-        for q >= 1
+
+        .. math::
+            W(q) = \\begin{cases}
+            \\frac{21}{2\\pi}(1-q)^4(4q+1)
+            &{\\rm for}\\;0 \\leq q < 1\\\\
+            0 &{\\rm for}\\;q \\geq 1
+            \\end{cases}
+
         Parameters
         ----------
-        q : np.array
+        q : array_like
             Dimensionless distance parameter.
+
         Returns
         -------
-        out : np.array
+        out : array_like
             Kernel value at positions q.
         """
 
@@ -202,21 +269,24 @@ class WendlandC2Kernel(_BaseSPHKernel):
         W *= (21 / 2 / np.pi)
         return W
 
-    def kernel_integral(self, dij, h):
+    def kernel_integral(self, dij, h, **kwargs):
         """
-        Calculate the kernel integral over a pixel. The formula used
-        approximates the kernel amplitude as constant across the pixel area and
-        converges to the true value within 1% for smoothing lengths >= 2
-        pixels.
+        Calculate the kernel integral over a pixel.
+
+        The formula used approximates the kernel amplitude as constant
+        across the pixel area and converges to the true value within 1%
+        for smoothing lengths >= 2 pixels.
+
         Parameters
         ----------
-        dij : astropy.units.Quantity, with dimensions of pixels
+        dij : Quantity, with dimensions of pixels
             Distances from pixel centre to particle positions, in pixels.
-        h : astropy.units.Quantity, with dimensions of pixels
+        h : Quantity, with dimensions of pixels
             Particle smoothing lengths (FWHM), in pixels.
+
         Returns
         -------
-        out : np.array
+        out : array_like
             Approximate kernel integral over the pixel area.
         """
 
@@ -233,44 +303,54 @@ class WendlandC2Kernel(_BaseSPHKernel):
         norm = 21 / 2 / np.pi
         return retval * norm / np.power(h, 2)
 
-    def validate(self, sm_lengths):
+    def validate(self, sm_lengths, noraise=False):
         """
         Check conditions for validity of kernel integral calculation.
+
         Parameters
         ----------
-        sm_lengths : astropy.units.Quantity, with dimensions of pixels
+        sm_lengths : Quantity, with dimensions of pixels
             Particle smoothing lengths (FWHM), in units of pixels.
         """
 
-        if (sm_lengths < 1.51 * U.pix).any():
-            raise RuntimeError("Martini.sph_kernels.WendlandC2Kernel.validate:"
-                               " SPH smoothing lengths must be >= 1.51 px in "
-                               "size for WendlandC2 kernel integral "
-                               "approximation accuracy within 1%. This check "
-                               "may be disabled by calling "
-                               "Martini.Martini.insert_source_in_cube with "
-                               "'skip_validation=True', but use this with"
-                               "care.")
+        valid = sm_lengths >= 1.51 * U.pix
+        if np.logical_not(valid).any():
+            self._validate_error(
+                "Martini.sph_kernels.WendlandC2Kernel.validate:\n"
+                "SPH smoothing lengths must be >= 1.51 px in "
+                "size for WendlandC2 kernel integral "
+                "approximation accuracy within 1%.\nThis check "
+                "may be disabled by calling "
+                "Martini.Martini.insert_source_in_cube with "
+                "'skip_validation=True', but use this with "
+                "care.\n",
+                sm_lengths,
+                valid,
+                noraise=noraise
+            )
 
-        return
+        return valid
 
 
 class WendlandC6Kernel(_BaseSPHKernel):
     """
     Implementation of the Wendland C6 kernel integral.
+
     The Wendland C6 kernel is used in the Magneticum code (not in
-    Gadget/Gadget2!). The exact integral is usually too slow to be practical,
-    and I have not yet undertaken the painful process of deriving a useful
-    approximation. Instead, use the GaussianKernel.mimic functionality.
+    Gadget/Gadget2!). The exact integral is usually too slow to be practical;
+    the implementation here approximates the kernel amplitude as constant
+    across the pixel, which converges to within 1% of the exact integral
+    provided the SPH smoothing lengths are at least 1.29 pixels in size.
+
     The WendlandC6 kernel is here defined as:
-        W(q) = (1365 / 64 / pi) * (1 - q)^8 * (1 + 8 * q + 25 * q^2 + 32 * q^3)
-        for 0 <= q < 1
-        W(q) = 0
-        for q >= 1
-    Returns
-    -------
-    out : WendlandC6Kernel
-        An appropriately initialized WendlandC6Kernel object.
+
+    .. math::
+        W(q) = \\begin{cases}
+        \\frac{1365}{64 \\pi} (1 - q)^8 (1 + 8q + 25q^2 + 32q^3)
+        &{\\rm for}\\;0 \\leq q < 1\\\\
+        0 &{\\rm for}\\;q \\geq 1
+        \\end{cases}
+
     """
 
     def __init__(self):
@@ -283,18 +363,24 @@ class WendlandC6Kernel(_BaseSPHKernel):
     def kernel(self, q):
         """
         Evaluate the kernel function.
+
         The WendlandC6 kernel is here defined as:
-        W(q) = (1365 / 64 / pi) * (1 - q)^8 * (1 + 8 * q + 25 * q^2 + 32 * q^3)
-        for 0 <= q < 1
-        W(q) = 0
-        for q >= 1
+
+        .. math::
+            W(q) = \\begin{cases}
+            \\frac{1365}{64 \\pi} (1 - q)^8 (1 + 8q + 25q^2 + 32q^3)
+            &{\\rm for}\\;0 \\leq q < 1\\\\
+            0 &{\\rm for}\\;q \\geq 1
+            \\end{cases}
+
         Parameters
         ----------
-        q : np.array
+        q : array_like
             Dimensionless distance parameter.
+
         Returns
         -------
-        out : np.array
+        out : array_like
             Kernel value at positions q.
         """
 
@@ -307,18 +393,20 @@ class WendlandC6Kernel(_BaseSPHKernel):
         W *= (1365 / 64 / np.pi)
         return W
 
-    def kernel_integral(self, dij, h):
+    def kernel_integral(self, dij, h, **kwargs):
         """
         Calculate the kernel integral over a pixel. Not currently implemented.
+
         Parameters
         ----------
-        dij : astropy.units.Quantity, with dimensions of pixels
+        dij : Quantity, with dimensions of pixels
             Distances from pixel centre to particle positions, in pixels.
-        h : astropy.units.Quantity, with dimensions of pixels
+        h : Quantity, with dimensions of pixels
             Particle smoothing lengths, in pixels.
+
         Returns
         -------
-        out : np.array
+        out : array_like
             Approximate kernel integral over the pixel area.
         """
 
@@ -395,46 +483,56 @@ class WendlandC6Kernel(_BaseSPHKernel):
         retval = retval / np.power(h, 2)
         return retval
 
-    def validate(self, sm_lengths):
+    def validate(self, sm_lengths, noraise=False):
         """
         Check conditions for validity of kernel integral calculation.
+
         Parameters
         ----------
-        sm_lengths : astropy.units.Quantity, with dimensions of pixels
+        sm_lengths : Quantity, with dimensions of pixels
             Particle smoothing lengths, in units of pixels.
         """
 
-        if (sm_lengths < 1.29 * U.pix).any():
-            raise RuntimeError("Martini.sph_kernels.WendlandC6Kernel.validate:"
-                               " SPH smoothing lengths must be >= 1.29 px in "
-                               "size for WendlandC2 kernel integral "
-                               "approximation accuracy within 1%. This check "
-                               "may be disabled by calling "
-                               "Martini.Martini.insert_source_in_cube with "
-                               "'skip_validation=True', but use this with"
-                               "care.")
-        return
+        valid = sm_lengths >= 1.29 * U.pix
+        if np.logical_not(valid).any():
+            self._validate_error(
+                "Martini.sph_kernels.WendlandC6Kernel.validate:\n"
+                " SPH smoothing lengths must be >= 1.29 px in "
+                "size for WendlandC6 kernel integral "
+                "approximation accuracy within 1%.\nThis check "
+                "may be disabled by calling "
+                "Martini.Martini.insert_source_in_cube with "
+                "'skip_validation=True', but use this with"
+                "care.",
+                sm_lengths,
+                valid,
+                noraise=noraise
+            )
+        return valid
 
 
 class CubicSplineKernel(_BaseSPHKernel):
     """
     Implementation of the cubic spline (M4) kernel integral.
+
     The cubic spline is the 'classic' SPH kernel. The exact integral is usually
     too slow to be practical; the implementation here approximates the kernel
     amplitude as constant across the pixel, which converges to within 1% of
-    the exact integral provided the SPH smoothing lengths are at least 2.5
+    the exact integral provided the SPH smoothing lengths are at least 1.16
     pixels in size.
+
     The cubic spline kernel is here defined as:
-        W(q) = (8 / pi) * (1 - 6 * q^2 * (1 - 0.5 * q))
-        for 0 <= q < 0.5
-        W(q) = (8 / pi) * 2 * (1 - q)^3
-        for 0.5 <= q < 1
-        W(q) = 0
-        for q >= 1
-    Returns
-    -------
-    out : CubicSplineKernel
-        An appropriately initialized CubicSplineKernel object.
+
+    .. math ::
+        W(q) = \\frac{8}{\\pi}\\begin{cases}
+        (1 - 6q^2(1 - \\frac{q}{2}))
+        &{\\rm for}\\;0 \\leq q < \\frac{1}{2}\\\\
+        2(1 - q)^3
+        &{\\rm for}\\;\\frac{1}{2} \\leq q < 1\\\\
+        0
+        &{\\rm for}\\;q \\geq 1
+        \\end{cases}
+
     """
 
     def __init__(self):
@@ -447,20 +545,27 @@ class CubicSplineKernel(_BaseSPHKernel):
     def kernel(self, q):
         """
         Evaluate the kernel function.
+
         The cubic spline kernel is here defined as:
-        W(q) = (8 / pi) * (1 - 6 * q^2 * (1 - 0.5 * q))
-        for 0 <= q < 0.5
-        W(q) = (8 / pi) * 2 * (1 - q)^3
-        for 0.5 <= q < 1
-        W(q) = 0
-        for q >= 1
+
+        .. math ::
+            W(q) = \\frac{8}{\\pi}\\begin{cases}
+            (1 - 6q^2(1 - \\frac{q}{2}))
+            &{\\rm for}\\;0 \\leq q < \\frac{1}{2}\\\\
+            2(1 - q)^3
+            &{\\rm for}\\;\\frac{1}{2} \\leq q < 1\\\\
+            0
+            &{\\rm for}\\;q \\geq 1
+            \\end{cases}
+
         Parameters
         ----------
-        q : np.array
+        q : array_like
             Dimensionless distance parameter.
+
         Returns
         -------
-        out : np.array
+        out : array_like
             Kernel value at positions q.
         """
 
@@ -473,21 +578,24 @@ class CubicSplineKernel(_BaseSPHKernel):
         W *= 8 / np.pi
         return W
 
-    def kernel_integral(self, dij, h):
+    def kernel_integral(self, dij, h, **kwargs):
         """
-        Calculate the kernel integral over a pixel. The formula used
-        approximates the kernel amplitude as constant across the pixel area and
-        converges to the true value within 1% for smoothing lengths >= 2.5
-        pixels.
+        Calculate the kernel integral over a pixel.
+
+        The formula used approximates the kernel amplitude as constant across
+        the pixel area and converges to the true value within 1% for smoothing
+        lengths >= 2.5 pixels.
+
         Parameters
         ----------
-        dij : astropy.units.Quantity, with dimensions of pixels
+        dij : Quantity, with dimensions of pixels
             Distances from pixel centre to particle positions, in pixels.
-        h : astropy.units.Quantity, with dimensions of pixels
+        h : Quantity, with dimensions of pixels
             Particle smoothing lengths, in pixels.
+
         Returns
         -------
-        out : np.array
+        out : array_like
             Approximate kernel integral over the pixel area.
         """
 
@@ -521,49 +629,61 @@ class CubicSplineKernel(_BaseSPHKernel):
         # rescaling from interval [0, 2) to [0, 1) requires mult. by 4
         return retval / 1.59689476201133 / np.power(h, 2) * 4
 
-    def validate(self, sm_lengths):
+    def validate(self, sm_lengths, noraise=False):
         """
         Check conditions for validity of kernel integral calculation.
+
         Parameters
         ----------
-        sm_lengths : astropy.units.Quantity, with dimensions of pixels
+        sm_lengths : Quantity, with dimensions of pixels
             Particle smoothing lengths, in units of pixels.
         """
 
-        if (sm_lengths < 1.16 * U.pix).any():
-            raise RuntimeError("Martini.sph_kernels.CubicSplineKernel.validate"
-                               ": SPH smoothing lengths must be >= 1.16 px in "
-                               "size for WendlandC2 kernel integral "
-                               "approximation accuracy within 1%. This check "
-                               "may be disabled by calling "
-                               "Martini.Martini.insert_source_in_cube with "
-                               "'skip_validation=True', but use this with"
-                               "care.")
-        return
+        valid = sm_lengths >= 1.16 * U.pix
+        if np.logical_not(valid).any():
+            self._validate_error(
+                "Martini.sph_kernels.CubicSplineKernel.validate:\n"
+                "SPH smoothing lengths must be >= 1.16 px in "
+                "size for CubicSplineKernel kernel integral "
+                "approximation accuracy within 1%.\nThis check "
+                "may be disabled by calling "
+                "Martini.Martini.insert_source_in_cube with "
+                "'skip_validation=True', but use this with "
+                "care.",
+                sm_lengths,
+                valid,
+                noraise=noraise
+            )
+        return valid
 
 
 class GaussianKernel(_BaseSPHKernel):
     """
     Implementation of a (truncated) Gaussian kernel integral.
+
     Calculates the kernel integral over a pixel. The 3 integrals (along dx,
     dy, dz) are evaluated exactly, however the truncation is implemented
     approximately, erring on the side of integrating slightly further than
     the truncation radius.
+
     The Gaussian kernel is here defined as:
-    W(q) = (sqrt(2 * pi) * sigma)^-3 * np.exp(-(q / sigma)^2 / 2)
-    for 0 <= q < truncate
-    W(q) = 0
-    for q >= truncate
-    with sigma=1/(2 * sqrt(2 * log(2))), s.t. FWHM = 1.
+
+    .. math::
+        W(q) = \\begin{cases}
+        (\\sqrt{2\\pi}\\sigma)^{-3}
+        \\exp\\left(-\\frac{1}{2}\\left(\\frac{q}{\\sigma}\\right)^2\\right)
+        &{\\rm for}\\;0 \\leq q < t\\\\
+        0 &{\\rm for}\\;q > t
+        \\end{cases}
+
+    with :math:`\\sigma=(2\\sqrt{2\\log(2)})^{-1}`, s.t. FWHM = 1, and
+    :math:`t` being the truncation radius.
+
     Parameters
     ----------
-    truncate : float
+    truncate : float, optional
         Number of standard deviations at which to truncate kernel (default=3).
         Truncation radii <2 may lead to large errors and are not recommended.
-    Returns
-    -------
-    out : GaussianKernel
-        An appropriately initialized GaussianKernel object.
     """
 
     def __init__(self, truncate=3):
@@ -577,19 +697,28 @@ class GaussianKernel(_BaseSPHKernel):
     def kernel(self, q):
         """
         Evaluate the kernel function.
+
         The Gaussian kernel is here defined as:
-        W(q) = (sqrt(2 * pi) * sigma)^-3 * np.exp(-(q / sigma)^2 / 2)
-        for 0 <= q < truncate
-        W(q) = 0
-        for q >= truncate
-        with sigma=1/(2 * sqrt(2 * log(2))), s.t. FWHM = 1.
+
+        .. math::
+            W(q) = \\begin{cases}
+            (\\sqrt{2\\pi}\\sigma)^{-3}
+            \\exp\\left(-\\frac{1}{2}\\left(\\frac{q}{\\sigma}\\right)^2\\right)
+            &{\\rm for}\\;0 \\leq q < t\\\\
+            0 &{\\rm for}\\;q > t
+            \\end{cases}
+
+        with :math:`\\sigma=(2\\sqrt{2\\log(2)})^{-1}`, s.t. FWHM = 1, and
+        :math:`t` being the truncation radius.
+
         Parameters
         ----------
-        q : np.array
+        q : array_like
             Dimensionless distance parameter.
+
         Returns
         -------
-        out : np.array
+        out : array_like
             Kernel value at positions q.
         """
 
@@ -601,21 +730,24 @@ class GaussianKernel(_BaseSPHKernel):
             np.zeros(q.shape)
         ) / self.norm
 
-    def kernel_integral(self, dij, h):
+    def kernel_integral(self, dij, h, **kwargs):
         """
-        Calculate the kernel integral over a pixel. The 3 integrals (along dx,
-        dy, dz) are evaluated exactly, however the truncation is implemented
-        approximately, erring on the side of integrating slightly further than
-        the truncation radius.
+        Calculate the kernel integral over a pixel.
+
+        The 3 integrals (along dx, dy, dz) are evaluated exactly, however the
+        truncation is implemented approximately, erring on the side of
+        integrating slightly further than the truncation radius.
+
         Parameters
         ----------
-        dij : astropy.units.Quantity, with dimensions of pixels
+        dij : Quantity, with dimensions of pixels
             Distances from pixel centre to particle positions, in pixels.
-        h : astropy.units.Quantity, with dimensions of pixels
+        h : Quantity, with dimensions of pixels
             Particle smoothing lengths (FWHM), in pixels.
+
         Returns
         -------
-        out : np.array
+        out : array_like
             Kernel integral over the pixel area.
         """
         sig = 1 / (2 * np.sqrt(2 * np.log(2)))  # s.t. FWHM = 1
@@ -646,12 +778,13 @@ class GaussianKernel(_BaseSPHKernel):
         retval /= self.norm
         return retval * h.unit ** -2
 
-    def validate(self, sm_lengths):
+    def validate(self, sm_lengths, noraise=False):
         """
         Check conditions for validity of kernel integral calculation.
+
         Parameters
         ----------
-        sm_lengths : astropy.units.Quantity, with dimensions of pixels
+        sm_lengths : Quantity, with dimensions of pixels
             Particle smoothing lengths (FWHM), in units of pixels.
         """
 
@@ -672,32 +805,43 @@ class GaussianKernel(_BaseSPHKernel):
             lims = None
             warn("GaussianKernel with truncation >=6sigma may unnecessarily "
                  "slow down computation.")
-        if (lims is not None) and \
-           (np.logical_and(sm_lengths > lims[0], sm_lengths < lims[1]).any()):
-            raise RuntimeError("Martini.sph_kernels.GaussianKernel.validate: "
-                               "SPH smoothing lengths must not be in interval "
-                               "[{0:.1f}, {1:.1f}] px ".format(*lims) +
-                               "for Gaussian kernel integral approximation "
-                               "accuracy within 1%. "
-                               "This check may be disabled by calling "
-                               "Martini.Martini.insert_source_in_cube with "
-                               "'skip_validation=True', but use this with"
-                               "care.")
-        return
+        if lims is None:
+            valid = np.ones(sm_lengths.shape, dtype=np.bool)
+        else:
+            valid = np.logical_or(
+                sm_lengths <= lims[0] * U.pix,
+                sm_lengths >= lims[1] * U.pix
+            )
+        if np.logical_not(valid).any():
+            self._validate_error(
+                "Martini.sph_kernels.GaussianKernel.validate:\n"
+                "SPH smoothing lengths must not be in interval "
+                "[{0:.1f}, {1:.1f}] px ".format(*lims) +
+                "for Gaussian kernel integral approximation "
+                "accuracy within 1%.\n"
+                "This check may be disabled by calling "
+                "Martini.Martini.insert_source_in_cube with "
+                "'skip_validation=True', but use this with"
+                "care.",
+                sm_lengths,
+                valid,
+                noraise=noraise
+            )
+        return valid
 
 
 class DiracDeltaKernel(_BaseSPHKernel):
     """
     Implementation of a Dirac-delta kernel integral.
+
     The Dirac-delta kernel is here defined as:
-    W(q) = inf
-    for q == 0
-    W(q) = 0
-    for q != 0
-    Returns
-    -------
-    out : DiracDeltaKernel
-        An appropriately initialized DiracDeltaKernel object.
+
+    .. math::
+        W(q) = \\begin{cases}
+        \\infty &{\\rm for}\\;q = 0\\\\
+        0 &{\\rm for}\\;q > 0
+        \\end{cases}
+
     """
 
     def __init__(self):
@@ -712,62 +856,210 @@ class DiracDeltaKernel(_BaseSPHKernel):
     def kernel(self, q):
         """
         Evaluate the kernel function.
+
         The Dirac-delta kernel is here defined as:
-        W(q) = inf
-        for q == 0
-        W(q) = 0
-        for q != 0
+
+        .. math::
+            W(q) = \\begin{cases}
+            \\infty &{\\rm for}\\;q = 0\\\\
+            0 &{\\rm for}\\;q > 0
+            \\end{cases}
+
         Parameters
         ----------
-        q : np.array
+        q : array_like
             Dimensionless distance parameter.
+
         Returns
         -------
-        out : np.array
+        out : array_like
             Kernel value at positions q.
         """
 
         return np.where(q, np.inf * np.ones(q.shape), np.zeros(q.shape))
 
-    def kernel_integral(self, dij, h):
+    def kernel_integral(self, dij, h, **kwargs):
         """
-        Calculate the kernel integral over a pixel. The particles are
-        approximated as point-like, ignoring any finite-sized kernel.
-        This is a reasonable approximation provided the smoothing
+        Calculate the kernel integral over a pixel.
+
+        The particles are approximated as point-like, ignoring any finite-sized
+        kernel. This is a reasonable approximation provided the smoothing
         length is < 0.5 pixel in size, ideally << 1 pixel in size.
+
         Parameters
         ----------
-        dij : astropy.units.Quantity, with dimensions of pixels
+        dij : Quantity, with dimensions of pixels
             Distances from pixel centre to particle positions, in pixels.
-        h : astropy.units.Quantity, with dimensions of pixels
+        h : Quantity, with dimensions of pixels
             Particle smoothing lengths (FWHM), in pixels.
+
         Returns
         -------
-        out : np.array
+        out : array_like
             Kernel integral over the pixel area.
         """
 
         return np.where((np.abs(dij) < 0.5 * U.pix).all(axis=0), 1, 0) \
             * U.pix ** -2
 
-    def validate(self, sm_lengths):
+    def validate(self, sm_lengths, noraise=False):
         """
         Check conditions for validity of kernel integral calculation.
+
         The Dirac-delta model approaches the exact integral when the smoothing
         length is << 1 pixel in size; at a minimum the smoothing length should
         be less than half the pixel size.
+
         Parameters
         ----------
-        sm_lengths : astropy.units.Quantity, with dimensions of pixels
+        sm_lengths : Quantity, with dimensions of pixels
             Particle smoothing lengths (FWHM), in units of pixels.
         """
 
-        if (sm_lengths > .5 * U.pix).any():
-            raise RuntimeError("Martini.sph_kernels.DiracDeltaKernel.validate:"
-                               " provided smoothing scale (FWHM) must be <= 1 "
-                               "px in size for DiracDelta kernel to be a "
-                               "reasonable approximation. Call "
-                               "Martini.Martini.insert_source_in_cube with "
-                               "'skip_validation=True' to override, at the "
-                               "cost of accuracy.")
+        valid = sm_lengths <= .5 * U.pix
+        if np.logical_not(valid).any():
+            self._validate_error(
+                "Martini.sph_kernels.DiracDeltaKernel.validate:\n"
+                "provided smoothing scale (FWHM) must be <= .5 "
+                "px in size for DiracDelta kernel to be a "
+                "reasonable approximation. Call "
+                "Martini.Martini.insert_source_in_cube with "
+                "'skip_validation=True' to override, at the "
+                "cost of accuracy.",
+                sm_lengths,
+                valid,
+                noraise=noraise
+            )
+        return valid
+
+
+class AdaptiveKernel(_BaseSPHKernel):
+    """
+    Allows use of multiple kernels to adapt to sph kernel-to-pixel size ratio.
+
+    Other provided kernels generally use approximations which break down if
+    the ratio of the pixel size and the sph smoothing length are above or below
+    some threshold. This (meta-)kernel accepts a list of other kernels in order
+    of decreasing priority. The validity of the approximations used in each
+    will be checked in turn and the first usable kernel for a given particle
+    will be used to smooth the particle onto the pixel grid. Note that the
+    initialized source and datacube instances are required as the smoothing
+    lengths and pixel sizes must be known at initialization of the
+    AdaptiveKernel module. Note that if `skip_validation` is used, any
+    particles with no valid kernel will default to the first kernel in the
+    list.
+
+    Parameters
+    ----------
+    kernels : iterable, containing classes inheriting from _BaseSPHKernel
+        Kernels to use, ordered by decreasing priority.
+
+    source : a martini.sources.SPHSource (or inheriting class) instance
+        The source to be used with this adaptive kernel.
+
+    datacube : a martini.DataCube instance
+        The datacube instance to be used with this adaptive kernel.
+    """
+
+    def __init__(self, kernels, source, datacube):
+        super().__init__()
+        # source.hsm_g is already required to be FWHM values
+        sm_length = np.arctan(
+            source.hsm_g / source.sky_coordinates.distance).to(
+                U.pix, U.pixel_scale(datacube.px_size / U.pix))
+        self.kernels = kernels
+        self.kernel_indices = -1 * np.ones(source.hsm_g.shape, dtype=np.int)
+        for ik, K in enumerate(kernels):
+            # if valid and not already assigned an earlier kernel, assign
+            self.kernel_indices[
+                np.logical_and(
+                    self.kernel_indices == -1,
+                    K.validate(sm_length * K._rescale, noraise=True)
+                )
+            ] = ik
+        _sizes_in_fwhm = np.array([K.size_in_fwhm for K in self.kernels])
+        self.size_in_fwhm = _sizes_in_fwhm[self.kernel_indices]
+        # ensure default is 0th entry
+        self.size_in_fwhm[self.kernel_indices == -1] = _sizes_in_fwhm[0]
+        _rescales = np.array([K._rescale for K in self.kernels])
+        self._rescale = _rescales[self.kernel_indices]
+        # ensure default is 0th entry
+        self._rescale[self.kernel_indices == -1] = _rescales[0]
+
+        return
+
+    def apply_mask(self, mask):
+        """
+        Apply mask to maskable attributes.
+
+        Parameters
+        ----------
+        mask : array_like containing booleans
+            Mask to apply to any maskable attributes.
+        """
+        self.size_in_fwhm = self.size_in_fwhm[mask]
+        self._rescale = self._rescale[mask]
+        self.kernel_indices = self.kernel_indices[mask]
+
+        return
+
+    def kernel(self, q):
+        raise NotImplementedError('AdaptiveKernel does not have an explicit '
+                                  'kernel function.')
+
+    def kernel_integral(self, dij, h, mask=np.s_[...]):
+        """
+        Calculate the kernel integral over a pixel.
+
+        Adaptively determines which kernel to use.
+
+        Parameters
+        ----------
+        dij : Quantity, with dimensions of pixels
+            Distances from pixel centre to particle positions, in pixels.
+        h : Quantity, with dimensions of pixels
+            Particle smoothing lengths (FWHM), in pixels.
+        mask : array_like, containing boolean array or slice
+
+        Returns
+        -------
+        out : array_like
+            Approximate kernel integral over the pixel area.
+        """
+
+        retval = np.zeros(h.shape) * h.unit ** -2
+        for ik in np.unique(self.kernel_indices[mask]):
+            K = self.kernels[0] if ik == -1 else self.kernels[ik]
+            kmask = self.kernel_indices[mask] == ik
+            retval[kmask] = K.kernel_integral(
+                dij[:, kmask],
+                h[kmask]
+            )
+        return retval
+
+    def validate(self, sm_lengths, noraise=False):
+        """
+        Check conditions for validity of kernel integral calculation.
+
+        Parameters
+        ----------
+        sm_lengths : Quantity, with dimensions of pixels
+            Particle smoothing lengths (FWHM), in units of pixels.
+        """
+
+        valid = self.kernel_indices >= 0
+        if np.logical_not(valid).any():
+            self._validate_error(
+                "Martini.sph_kernels.AdaptiveKernel.validate:\n"
+                "Some particles have no kernel candidate for which "
+                "accuracy passes validation.\nThis check "
+                "may be disabled by calling "
+                "Martini.Martini.insert_source_in_cube with "
+                "'skip_validation=True', but use this with "
+                "care.\n",
+                sm_lengths,
+                valid,
+                noraise=noraise
+            )
+
         return
